<?php
// $Id: kdbrunidentifier.php 8097 2010-07-15 09:57:24Z s_voec01 $
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

ADEI::RequireClass('KDBConstants', TRUE);

class KDBRunIdentifier
{
    public $system = kSystemInvalid;
    public $number = 0;
    public $subRun = 0;
    
    function __construct()
    {
        if (func_num_args() == 1 && is_string(func_get_arg(0))) {
            $this->ParseString(func_get_arg(0));
        }
        else {
            if (func_num_args() >= 1) {
                $this->system = func_get_arg(0);
            }
            if (func_num_args() >= 2) {
                $this->number = func_get_arg(1);
            }
            if (func_num_args() >= 3) {
                $this->subRun = func_get_arg(2);
            }
        }
    }

    function GetRun()
    {
        return $this->number;
    }

    function GetSubRun()
    {
        return $this->subRun;
    }

    function SetSubRun($subrun)
    {
        $this->subRun = $subrun;
    }

    function GetSystem()
    {
        return $this->system;
    }

    function GetRunName()
    {
        return sprintf("%s%08d", $this->GetSystemName(), $this->number);
    }

    function GetName()
    {
        $name = $this->GetRunName();
        if ($this->subRun != 0) {
            $name .= sprintf(".%05d", $this->subRun);
        }

        return $name;
    }

    function IsValid()
    {
        return $this->system != kSystemInvalid && $this->number != 0;
    }

    private function ParseString($text)
    {
        $pattern = '=^(?<system>[a-z]+)(?<run>\d+)(\.(?<subrun>\d+))?=';
        
        if (preg_match($pattern, $text, $matches)) {
            try {
                $this->system =
                    KDBConstants::OrcaSystemValue($matches['system']);
            }
            catch (ADEIException $e) {
                $this->system = kSystemInvalid;
            }
            
            $this->number = (int) $matches['run'];
            if (array_key_exists('subrun', $matches)) {
                $this->subRun = (int) $matches['subrun'];
            }
        }
    }

    private function GetSystemName()
    {
        return KDBConstants::OrcaSystemName($this->system);
    }
}
?>